#include "eBot_Sim_Predef.h"
#include "adacrus.h"
#define UC unsigned char

using namespace std;

Adacrus__main_mem mem;

/**
 * @brief      Velocity function with direction control. You can enter negative values to 
 change the wheel in backwards direction.
 */
void improved_velocity(int left_vel, int right_vel) {
    right_vel = right_vel > 255 ? 255 : right_vel;
    right_vel = right_vel < -255 ? -255 : right_vel;
    left_vel = left_vel > 255 ? 255 : left_vel;
    left_vel = left_vel < -255 ? -255 : left_vel;
    if (left_vel >= 0 && right_vel >= 0) {
        forward();
        velocity(left_vel, right_vel);
    } else if (left_vel < 0 && right_vel >= 0) {
        left();
        velocity(-left_vel, right_vel);
    } else if (left_vel >= 0 && right_vel < 0) {
        right();
        velocity(left_vel, -right_vel);
    } else
        stop();
}


void traverse_line_to_goal() {
    UC left_wl_reading = 0, right_wl_reading = 0, center_wl_reading = 0, fr_proximity_reading = 0;
    Adacrus__main_out _res;
    Adacrus__main_reset(&mem);
    while (true) {
        // read sensor values
        left_wl_reading = convert_analog_channel_data( left_wl_sensor_channel );
        center_wl_reading = convert_analog_channel_data( center_wl_sensor_channel );
        right_wl_reading = convert_analog_channel_data( right_wl_sensor_channel );
        fr_proximity_reading = convert_analog_channel_data( ir_prox_5_sensor_channel );
        // step - generate velocity outputs for the wheels
        Adacrus__main_step(left_wl_reading, center_wl_reading, right_wl_reading, fr_proximity_reading,
                           &_res, &mem);
        // actuate
        improved_velocity(_res.v_l, _res.v_r);
        _delay_ms(5);
    }
}
